; Disassembly of the FPE init file (driver) for the Apple IIgs
; by Axel Muhr - see www.geekdot.com/number-cruncher for more detail

; V1.0 - 1.12.2016

; yesterday I learned how to use the MC6888x as an 8 bit peripheral

: only four address lines, A0 to A3, are used to address the MC6888x. this means that
; the 32 addresses of the 6888x are mapped to only 16 addresses and this is done by
; the Xilinx FPGA
;
; register	MC6888x		apple			comment
; --------	-------		-----			-------
; FP_Stat	+$00 to +$01	+$00 to +$01	r  response register
; FP_Ctrl	+$02 to +$03	+$02 to +$03	 w control register
; FP_Save	+$04 to +$05	+$04 to +$05	r  save register
; FP_Rest	+$06 to +$07	+$06 to +$07	rw restore register
; FP_Cmd	+$0A to +$0B	+$08 to +$09	 w command register
; FP_Ccr	+$0E to +$0F	+$0A to +$0B	 w condition code register
; FP_Op	+$10 to +$13	+$0C to +$0F	rw operand register
;

; MC6888x FPU register definitions

FP_base		EQU	$E1C090		; FPU base address
FP_Stat		EQU	FP_base+$00		; r  response register
; FP_Ctrl		EQU	FP_base+$02		;  w control register
; FP_Save		EQU	FP_base+$04		; r  save register
; FP_Rest		EQU	FP_base+$06		; rw restore register
FP_Cmd		EQU	FP_base+$08		;  w command register
FP_Cmdl		EQU	FP_base+$09		;  w command register, low byte
; FP_Ccr		EQU	FP_base+$0A		;  w condition code register
FP_Oph		EQU	FP_base+$0C		; rw operand register, high word
FP_Opl		EQU	FP_base+$0E		; rw operand register, low word

; reading the FP_Stat register is used to initiate execution of commands

; most FPU command words are made up as follows ..
;
; 0r0sssdddccccccc
;  | \_/\_/\_____/
;  |  |  |    |
;  |  |  |    +----- opcode
;  |  |  +---------- destination
;  |  +------------- source
;  +---------------- operand/register
;
; when r=1 and sss=111 the opcode becomes the address for a constant from the internal
; ROM to be moved to the destination register
;
; because the 65xx(x) series are little endian and the 68xxx series big endian the bytes
; of the command word need to be swapped to get the final immediate value. the same is
; true when comparing expected values with the response register. also there is a lot of
;
;	lda	<memory>		; get word
;	xba				; swap bytes
;	sta	<FPU>			; save FPU operand
;
; and
;
;	lda	<FPU>			; get FPU operand
;	xba				; swap bytes
;	sta	<memory>		; save word

; MC6888x data formats ..
;
; E = eeee = exponent bits
; M = mmmm = mantissa bits
; s        = sign bit
; b        = binary bit
;
; extended precision
;
;	seee E E E 0000 0000 0000 0000 m.mmm M M M M M M M M M M M M M M M
;
;  takes three longword reads
;
; double precision
;
;	seee E E M M M M M M M M M M M M M
;
;  takes two longword reads
;
; single precision
;
;	seee E emmm M M M M M
;
;  takes one longword read
;
; long integer
;
;	bbbb bbbb bbbb bbbb bbbb bbbb bbbb bbbb
;
;  takes one longword read
;
; word integer
;
;	bbbb bbbb bbbb bbbb
;
;  takes one word read from the operand register high word

;------------------------------------------------------------------------------------
;
; ??

	clc				; clear carry to enter native mode
	xce				; swap Cb to Xb, set processor mode
	php				; save processor status
	LONG	I,M			; set 16 bit register and memory mode

	pea	$0000			; push parameters on stack
	pea	$0000			; for getting pointer table for system
	pea	$0000			; tools.
	pea	$000A			; In this case: SANE
	_GetTSPtr			; call GetToolSetPointer
	pla				; pull pointer low word
	sta	$F0			; save pointer low word
	pla				; pull pointer high word
	sta	$F2			; save pointer high word
	ldy	#$0026		; set index to ??
	lda	[$F0],y		; get ?? high word
	sta	>$0143		; save ?? high word
	ldy	#$0024		; set index to ??
	lda	[$F0],y		; get ?? low word
	sta	>$0145		; save ?? low word
	clc				; clear carry for add
	lda	>$0145		; get ?? low word
	adc	#$0001		; add 1 ??
	sta	>$0145		; save incremented ?? low word
	lda	>$0143		; get ?? high word
	adc	#$0000		; add carry from low word
	tay				; copy incremented ?? high word
	lda	>$0145		; read incremented ?? low word
	sta	>$0143		; save to ?? low word
	tya				; copy incremented ?? high word back
	sta	>$0145		; save to ?? high word

	pea	$0000
	pea	$000A
	pea	LAB_006E|-$10
	pea	LAB_006E
	_SetTSPtr
	_SetDefaultTPT
	plp
	xce
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_006E
	.byte	$0C,$00,$00,$00

	.word	$009D
	.byte	$00,$00
	.word	LAB_00A9-1
	.byte	$00,$00
	.byte	$00,$00,$00,$00
	.byte	$00,$00,$00,$00

	.word	$0103
	.byte	$00,$00
	.byte	$00,$00,$00,$00
	.byte	$00,$00,$00,$00
	.byte	$00,$00,$00,$00

	.word	LAB_0114-1
	.byte	$00,$00
	.byte	$00,$00,$00,$00
	.word	LAB_01B9-1
	.word	$00,$00
	.word	LAB_00A1-1
	.byte	$00

;------------------------------------------------------------------------------------
;
; ??

LAB_00A1
	ldx	#$0000
	clc
	lda	#$0000
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_00A9
	txy
	lda	$07,s
	phd				; push direct page register
	tcd				; transfer A to direct page register
	lda	#$0000
	ldx	#$00FF
LAB_00B4
	sta	$00,x
	dex
	dex
	bpl	LAB_00B4

	lda	#$0088		; FMOVE to FPSR
	sta	>FP_Cmd		; save command word
LAB_00C1
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_00C1		; loop if FPU busy

	lda	#$0000		; clear A
	sta	>FP_Oph		; save operand high word
	sta	>FP_Opl		; save operand low word
LAB_00D5
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_00D5		; loop if FPU busy

	pld				; restore direct page register
	tyx
	lda	$07,s
	pea	$0000
	phx
	pea	$0000
	pha
	lda	$05,s
	and	#$00FF
	sta	$05,s
	_SetWAP
	lda	$05,s
	sta	$07,s
	lda	$03,s
	sta	$05,s
	lda	$01,s
	sta	$03,s
	pla
	lda	#$0000		; clear A, NULL restore frame, device reset
	sta	>FP_Rest		; abort any instruction and prepare for restore
	lda	>FP_Rest		; mandatory read back of restore format word
	lda	#$0000
	clc
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_0114
	tay
	lda	$07,s
	tax
	and	#$FF00
	cmp	#$0500
	beq	LAB_0141
	txa
	and	#$00FF
	cmp	#$0009
	beq	LAB_0141
	cmp	#$000B
	beq	LAB_0141
	cmp	#$001C
	beq	LAB_0141
	cmp	#$0011
	beq	LAB_0141
	cmp	#$001E
	bne	LAB_013F
	bra	LAB_0141

LAB_013F
	bra	LAB_0147

LAB_0141
	tya
	jmp	>$000000

;LAB_0146
	.byte	$00

LAB_0147
	TYA

	phd				; push direct page register
	tcd				; transfer A to direct page register
	SHORT	M			; set 8 bit memory mode
	phb
	pla
	sta	$FA
	phk
	plb
	LONG	M			; set 16 bit memory mode
	lda	$09,s
	tax
	sta	$0A
	and	#$00FF
	sta	$2A
	txa
	and	#$FF00
	sta	$2E
	lda	$03,s
	sta	$00
	lda	$05,s
	sta	$02
	lda	$07,s
	sta	$04

	lda	$2A
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_0179,x		; get floating point vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; floating point math functions

LAB_0179
	.word	LAB_0246-1		;  0 FADD
	.word	LAB_1988-1		;  1 FOSETENV		??
	.word	LAB_1AD8-1		;  2 FSUB
	.word	LAB_146A-1		;  3 FOGETENV		??
	.word	LAB_14F9-1		;  4 FMUL
	.word	LAB_1A12-1		;  5 FOSETHV		??
	.word	LAB_0C55-1		;  6 FDIV
	.word	LAB_148D-1		;  7 FOGETHV		??
	.word	LAB_0786-1		;  8 FCMP
	.word	LAB_0A47-1		;  9 FOD2B			??
	.word	LAB_0786-1		; 10 FCMP			[FOCPX ??]
	.word	LAB_0411-1		; 11 FOB2D			??
	.word	LAB_16DB-1		; 12 FREM
	.word	LAB_1640-1		; 13 FNEG
	.word	LAB_1CFE-1		; 14 FMOVE to FP1		[FOZ2X ??]
	.word	LAB_01F2-1		; 15 FABS
	.word	LAB_1BDF-1		; 16 FX2Z
	.word	LAB_0A14-1		; 17 FCOPYSGN
	.word	LAB_1A27-1		; 18 FSQRT
	.word	$0000			; 19 FONEXT			??
	.word	LAB_18C1-1		; 20 FINT
	.word	LAB_16BE-1		; 21 FOSETXCP		??
	.word	LAB_1B5B-1		; 22 FINTRZ			[FOTTI ??]
	.word	LAB_16AC-1		; 23 FOPROCENTRY		??
	.word	LAB_1915-1		; 24 FOSCALB		??
	.word	LAB_1694-1		; 25 FOPROCEXIT		??
	.word	LAB_14A5-1		; 26 FLOG2
	.word	LAB_16C8-1		; 27 FOTESTXCP		??
	.word	LAB_0855-1		; 28 FOCLASS		??
	.word	$0000
	.word	$0000
	.word	$0000


;------------------------------------------------------------------------------------
;
; ??

LAB_01B9
	TAY
	phd				; push direct page register
	tcd				; transfer A to direct page register
	SHORT	M			; set 8 bit memory mode
	phb
	pla
	sta	$FA
	phk
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	lda	$07,s
	tax
	tya
	lda	LAB_01D0,x		; get transcendental function vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; transcendental functions

LAB_01D0
	.word	LAB_136D-1		; FLOGN
	.word	LAB_1377-1		; FLOG2
	.word	LAB_1381-1		; FLOGNP1
	.word	LAB_09B9-1		; FLOG2P1
	.word	LAB_138B-1		; FETOX
	.word	LAB_1395-1		; FTWOTOX
	.word	LAB_139F-1		; FETOXM1
	.word	LAB_09B1-1		; FOEXP21X		??
	.word	LAB_0A04-1		; FOXPWRI		??
	.word	LAB_0A0C-1		; FOXPWRY		??
	.word	LAB_09C1-1		; FOCOMPOUND	??
	.word	LAB_09C1-1		; FOANNUITY		??
	.word	LAB_13A9-1		; FATAN
	.word	LAB_13B3-1		; FSIN
	.word	LAB_13BD-1		; FCOS
	.word	LAB_13C7-1		; FTAN
	.word	LAB_09A9-1		; FRAND

;------------------------------------------------------------------------------------
;
; FABS

LAB_01F2
	JSR	LAB_1347
	LDA	#$9848		; FABS, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_01FC
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_01FC		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_023B
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_023B		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FADD

LAB_0246
	jsr	LAB_03E1
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0250
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0250		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_028F
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_028F		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_03D7,x		; get FADD precision vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FADD extended precision source

LAB_02A1
	lda	#$A248		; FADD, extended precision, destination FP1

;------------------------------------------------------------------------------------
;
; do FPU command, extended precision source

LAB_02A4
	sta	>FP_Cmd		; save command word
LAB_02A8
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_02A8		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FADD double precision source

LAB_02EA
	lda	#$A254		; FADD, double precision, destination FP1

;------------------------------------------------------------------------------------
;
; do FPU command, double precision source

LAB_02ED
	sta	>FP_Cmd		; save command word
LAB_02F1
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_02F1		; loop if FPU busy

	ldy	#$0006		; index to double precision sign and exponent
	lda	[$10],y		; get double precision sign and exponent
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	ldy	#$0004		; index to double precision high word
	lda	[$10],y		; get double precision high word
	xba				; swap bytes
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0002		; index to double precision next high word
	lda	[$10],y		; get double precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	lda	[$10]			; get double precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FADD single precision source

LAB_0322
	lda	#$A244		; FADD, single precision, destination FP1

;------------------------------------------------------------------------------------
;
; do FPU command, single precision source

LAB_0325
	sta	>FP_Cmd		; save command word
LAB_0329
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0329		; loop if FPU busy

	ldy	#$0002		; set index to single precision high word
	lda	[$10],y		; get single precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand high word

	lda	[$10]			; get single precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save to FPU operand low word
	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FADD long integer source

LAB_0346
	lda	#$A240		; FADD, long integer, destination FP1

;------------------------------------------------------------------------------------
;
; do FPU command, long integer source

LAB_0349
	sta	>FP_Cmd		; save command word
LAB_034D
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_034D		; loop if FPU busy

	ldy	#$0002		; set index to high word
	lda	[$10],y		; get integer high word
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand high word

	lda	[$10]			; get integer low word
	xba				; swap bytes
	sta	>FP_Opl		; save to FPU operand low word
	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FADD word integer source

LAB_036A
	lda	#$A250		; FADD, word integer, destination FP1

;------------------------------------------------------------------------------------
;
; do FPU command, word integer source

LAB_036D
	sta	>FP_Cmd		; save command word
LAB_0371
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0371		; loop if FPU busy

	lda	[$10]			; get word integer source
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand word

;------------------------------------------------------------------------------------
;
; wait for FPU and get extended result

LAB_0381
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0381		; loop if FPU busy

	lda	#$8068		; FMOVE from FPn, extended precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_0390
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0390		; loop if FPU busy

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0008		; index to extended precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; second word is all 0s and is discarded

	lda	>FP_Oph		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0006		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Opl		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word
LAB_03CC
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_03CC		; loop if FPU busy

	brl	LAB_08CC		; read/set FP status bits ??

;------------------------------------------------------------------------------------
;
; FADD precision vectors

LAB_03D7
	.word	LAB_02A1-1		; FADD extended precision source
	.word	LAB_02EA-1		; FADD double precision source
	.word	LAB_0322-1		; FADD single precision source
	.word	LAB_0346-1		; FADD long integer source
	.word	LAB_036A-1		; FADD word integer source

;------------------------------------------------------------------------------------
;
; ??

LAB_03E1
	PLA				; pull return address
	sta	$30			; save it

	lda	$11,s
	sta	$12
	lda	$0F,s
	sta	$10
	lda	$0D,s
	sta	$0E
	lda	$0B,s
	sta	$0C
	lda	$07,s
	sta	$11,s
	sta	$04
	lda	$05,s
	sta	$0F,s
	sta	$02
	lda	$03,s
	sta	$0D,s
	sta	$00
	pla
	sta	$09,s
	pla
	pla
	pla
	pla

	lda	$30			; get return address
	pha				; put it back on the stack
	rts

;------------------------------------------------------------------------------------
;
; ??

LAB_0411
	jsr	LAB_1D33
	SHORT	I,M			; set 8 bit register and memory mode
	ldx	$2F			; get precision index
	lda	LAB_0781,x		; get command word high byte
	sta	>FP_Cmd		; save command word high byte
	lda	#$80			; FMOVE to FPn, precision set by high byte, destination FP1
	sta	>FP_Cmdl		; save command word low byte
	LONG	I,M			; set 16 bit register and memory mode
LAB_0427
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0427		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	tax
	bne	LAB_046F

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

	brl	LAB_04BA

LAB_046F
	dex
	bne	LAB_049A

	ldy	#$0006		; index to double precision sign and exponent
	lda	[$10],y		; get double precision sign and exponent
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	ldy	#$0004		; index to double precision high word
	lda	[$10],y		; get double precision high word
	xba				; swap bytes
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0002		; index to double precision next high word
	lda	[$10],y		; get double precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	lda	[$10]			; get double precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	brl	LAB_04BA

LAB_049A
	cpx	#$0003
	bne	LAB_04A9
	lda	[$10]
	xba				; swap bytes
	sta	>FP_Oph
	brl	LAB_04BA

LAB_04A9
	ldy	#$0002
	lda	[$10],y
	xba				; swap bytes
	sta	>FP_Oph
	lda	[$10]
	xba				; swap bytes
	sta	>FP_Opl
LAB_04BA
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_04BA		; loop if FPU busy

	SHORT	I,M			; set 8 bit register and memory mode
	ldy	#$02
	lda	[$14],y
	sta	$D8
	lda	[$14]
	bne	LAB_04D2
	lda	$D8
	bra	LAB_04F1

LAB_04D2
	ldy	#$02
	lda	$D8
	cmp	#$00
	beq	LAB_04F5
	bmi	LAB_04EB
	cmp	#$12
	bmi	LAB_04E4
	lda	#$EF
	sta	[$14],y
LAB_04E4
	sec
	lda	#$00
	sbc	[$14],y
	bra	LAB_04F1

LAB_04EB
	cmp	#$F0
	bpl	LAB_04F1
	lda	#$F0
LAB_04F1
	and	#$7F
	sta	[$14],y
LAB_04F5
	SHORT	M			; set 8 bit memory mode
	lda	#$6C			; FMOVE from FPn, packed decimal real, source FP1
	sta	>FP_Cmd		; save command word high byte
	lda	#$80			; set command low byte
	ora	[$14],y		; OR in static k-factor
					; -64 to 0	number of significant digits after the decimal
					; +1 to +17	number of significant digits in the mantissa
					; +18 up	treated as +17, sets the OPERR bit
	sta	>FP_Cmdl		; save command word low byte
	LONG	M			; set 16 bit memory mode
LAB_0507
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0507		; loop if FPU busy

	lda	>FP_Oph
	sta	$82
	lda	>FP_Opl
	sta	$84
	lda	>FP_Oph
	sta	$86
	lda	>FP_Opl
	sta	$88
	lda	>FP_Oph
	sta	$8A
	lda	>FP_Opl
	sta	$8C
LAB_0534
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0534		; loop if FPU busy

	SHORT	I,M			; set 8 bit register and memory mode
	lda	$82
	tay
	and	#$30
	beq	LAB_0548
	brl	LAB_05B5

LAB_0548
	lda	$82
	bit	LAB_077F
	beq	LAB_0557
	ldx	#$01
	stx	LAB_074F
	brl	LAB_055C

LAB_0557
	ldx	#$00
	stx	LAB_074F
LAB_055C
	jsr	LAB_06C6
	lda	#$11
	sta	$D4
	ldx	#$04
	inx
	lda	$85
	and	#$0F
	ora	#$30
	sta	LAB_074F,x
	ldy	#$04
LAB_0571
	cpx	$D4
	bne	LAB_0578
	brl	LAB_05A1

LAB_0578
	inx
	phx
	tya
	tax
	lda	$82,x
	sta	$F2
	plx
	and	#$F0
	clc
	ror	a
	ror	a
	ror	a
	ror	a
	ora	#$30
	sta	LAB_074F,x
	cpx	$D4
	bne	LAB_0593
	beq	LAB_05A1

LAB_0593
	inx
	lda	$F2
	and	#$0F
	ora	#$30
	sta	LAB_074F,x
	iny
	brl	LAB_0571

LAB_05A1
	lda	[$14]
	beq	LAB_05B2
	LONG	M			; set 16 bit memory mode
	lda	#$0000
	sec
	sbc	$D8
	sta	LAB_0751
	SHORT	M			; set 8 bit memory mode
LAB_05B2
	brl	LAB_05F5

LAB_05B5
	lda	$83
	ldx	#$03
LAB_05B9
	lda	$82,x
	bne	LAB_05CA
	inx
	cpx	#$0B
	bne	LAB_05B9
	brl	LAB_05E4

;------------------------------------------------------------------------------------
;
; ??

LAB_05C5
	.byte	$4E,$34,$30,$30,$30

;------------------------------------------------------------------------------------
;
; ??

LAB_05CA
	lda	#$05
	ldx	#$04
	sta	LAB_074F,x
	inx
	ldy	#$00
LAB_05D4
	lda	LAB_05C5,y
	sta	LAB_074F,x
	inx
	iny
	cpy	#$05
	bne	LAB_05D4
	brl	LAB_0691

;------------------------------------------------------------------------------------
;
; ??

LAB_05E3
	.byte	$49

;------------------------------------------------------------------------------------
;
; ??

LAB_05E4
	LDA	#$01
	LDX	#$04
	STA	LAB_074F,x
	INX
	LDA	LAB_05E3

	sta	LAB_074F,x
	brl	LAB_0691

LAB_05F5
	lda	[$14]
	bne	LAB_05FC
	brl	LAB_0649

LAB_05FC
	lda	$82
	bit	LAB_0780
	beq	LAB_0626
	lda	$D8
	sec
	sbc	$83
	bmi	LAB_0611
	inc	a
	sta	LAB_0753
	brl	LAB_0691

LAB_0611
	ldx	#$01
	stx	LAB_0753
	ldy	$D8
	lda	#$30
	ldx	#$05
LAB_061C
	sta	LAB_074F,x
	inx
	dey
	bpl	LAB_061C
	brl	LAB_0691

LAB_0626
	lda	$DB
	bne	LAB_0678
	lda	$DA
	beq	LAB_0640
	sec
	sbc	#$11
	bpl	LAB_0678
	inc	$DA
	lda	$D8
	clc
	adc	$DA
	sta	LAB_0753
	brl	LAB_0691

LAB_0640
	ldx	$D8
	inx
	stx	LAB_0753
	brl	LAB_0691

LAB_0649
	ldx	$D8
	inx
	stx	LAB_0753
	dex
	dex
	stx	$D8
	LONG	M			; set 16 bit memory mode
	lda	$82
	and	#$0040
	bne	LAB_0668
	dec	$DA
	lda	$DA
	sec
	sbc	$D8
	sta	LAB_0751
	bra	LAB_0673

LAB_0668
	lda	#$0000
	sec
	sbc	$DA
	sbc	$D8
	sta	LAB_0751
LAB_0673
	SHORT	M			; set 8 bit memory mode
	brl	LAB_0691

LAB_0678
	ldy	#$02
	lda	[$14],y
	sec
	sbc	LAB_0751
	sta	LAB_0751
	iny
	lda	[$14],y
	sbc	LAB_0752
	sta	LAB_0752
	lda	#$11
	sta	LAB_0753
LAB_0691
	lda	$D8
	ldy	#$02
	sta	[$14],y
	iny
	lda	$D9
	sta	[$14],y
	ldy	#$04
LAB_069E
	lda	LAB_074F,y
	sta	[$0C],y
	dey
	bpl	LAB_069E

	ldy	#$05
	ldx	LAB_0753
LAB_06AB
	lda	LAB_074F,y
	sta	[$0C],y
	iny
	dex
	bpl	LAB_06AB

	ldx	$D8
LAB_06B6
	lda	LAB_074F,y
	sta	[$0C],y
	iny
	dex
	bpl	LAB_06B6
	lda	#$00
	tay
	tax
	brl	LAB_0724

LAB_06C6
	ldy	#$00
	sty	LAB_0750
	sty	LAB_0751
	sty	LAB_0752
	sty	$DA
	sty	$DB
	lda	$82
	and	#$0F
	beq	LAB_06E4

	tax
	lda	LAB_0731,x		; get 100s high byte
	sta	LAB_0752
	sta	$DB
LAB_06E4
	lda	$83
	tay
	and	#$0F
	sta	LAB_0751
	sta	$DA
	tya
	and	#$F0
	beq	LAB_0704

	clc
	ror	a
	ror	a
	ror	a
	ror	a
	tax
	lda	LAB_0745,x		; get 10s
	adc	LAB_0751
	sta	LAB_0751
	sta	$DA
LAB_0704
	lda	$82
	and	#$0F
	beq	LAB_0723

	ldy	#$00
	lda	LAB_073B,x		; get 100s low byte
	adc	LAB_0751
	bcc	LAB_0715
	iny
LAB_0715
	sta	LAB_0751
	tya
	beq	LAB_0723
	adc	LAB_0752
	sta	LAB_0752
	sta	$DA
LAB_0723
	rts

LAB_0724
	lda	$FA
	pha
	plb
	LONG	I,M			; set 16 bit register and memory mode
	pld				; restore direct page register
	lda	#$0000
	tay
	tax
	rtl

;------------------------------------------------------------------------------------
;
; 100s high bytes

LAB_0731
	.byte	$00,$00,$00,$01,$01,$01,$02,$02,$03,$03

; 100s low bytes

LAB_073B
	.byte	$00,$64,$C8,$2C,$90,$F4,$58,$6C,$20,$84

; 10s

LAB_0745
	.byte	$00,$0A,$14,$1E,$28,$32,$3C,$46,$50,$5A

;------------------------------------------------------------------------------------
;
; ??

LAB_074F
	.byte	$00,$00
LAB_0751
	.byte	$00,$00
LAB_0753
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30
	.byte	$30,$30,$30,$30

LAB_077F
	.byte	$80,$40

;------------------------------------------------------------------------------------
;
; high bytes for FMOVE to FPn

LAB_0781
	.byte	$48			; extended precision
	.byte	$54			; double precision
	.byte	$44			; single precision
	.byte	$40			; long integer
	.byte	$50			; word integer

;------------------------------------------------------------------------------------
;
; FCMP

LAB_0786
	JSR	LAB_03E1

	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0790
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0790		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_07CF
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_07CF		; loop if FPU busy

	SHORT	I,M			; set 8 bit register and memory mode
	ldx	$2F			; get precision index
	lda	LAB_08C7,x		; get command word high byte
	sta	>FP_Cmd		; save command word high byte
	lda	#$B8			; FCMP, precision set by high byte, destination FP1
	sta	>FP_Cmdl		; save command word low byte
	jsr	LAB_157C		; save operand of precision X
	SHORT	I,M			; set 8 bit register and memory mode
	lda	#$00
	sta	$1E
	lda	$FA
	pha
	plb
	lda	$35
	bit	#$04
	beq	LAB_0802
	ldx	#$02
	ldy	#$00
	brl	LAB_082E

LAB_0802
	bit	$01
	beq	LAB_081E
	lda	$33
	bit	#$40
	beq	LAB_0818
	lda	$0A
	cmp	#$0A
	bne	LAB_0818
	lda	#$01
	ora	$1D
	ora	$1E
LAB_0818
	ldx	#$01
	ldy	#$01
	bra	LAB_082E

LAB_081E
	lda	$35
	bit	#$08
	bne	LAB_082A
	ldx	#$80
	ldy	#$80
	bra	LAB_082E

LAB_082A
	ldx	#$40
	ldy	#$40
LAB_082E
	LONG	I,M			; set 16 bit register and memory mode
	pld				; restore direct page register
	txa
	cmp	#$0001
	bne	LAB_083C
	SHORT
	rep	#$C0
	rtl

LAB_083C
	cmp	#$0002
	bne	LAB_0846
	sep	#$02
	rep	#$C0
	rtl

LAB_0846
	cmp	#$0080
	bne	LAB_0850
	sep	#$80
	rep	#$42
	rtl

LAB_0850
	sep	#$40
	rep	#$82
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_0855
	jsr	LAB_1347
	lda	$0C
	sta	$10
	lda	$0E
	sta	$12
	SHORT	I,M			; set 8 bit register and memory mode
	ldx	$2F			; get precision index
	lda	LAB_08C7,x		; get command word high byte
	sta	>FP_Cmd		; save command word high byte
	lda	#$3A			; FTST, precision set by high byte
	sta	>FP_Cmdl		; save command word low byte
	jsr	LAB_157C		; save operand of precision X
	SHORT	I,M			; set 8 bit register and memory mode
	lda	$FA
	pha
	plb
	ldy	#$00
	lda	$35
	and	#$07
	beq	LAB_08AF
	bit	#$04
	beq	LAB_088B
	ldx	#$FF
	brl	LAB_08B5

LAB_088B
	bit	#$01
	beq	LAB_08A3
	lda	$33
	bne	LAB_08AB
	ldx	#$FD
	brl	LAB_08B5

;LAB_0898
	bit	#$08
	beq	LAB_08A3
	ldx	#$00
	ldy	#$80
	brl	LAB_08BF

LAB_08A3
	bit	#$02
	beq	LAB_08AB
	ldx	#$FE
	bra	LAB_08B5

LAB_08AB
	ldx	#$FC
	bra	LAB_08B5

LAB_08AF
	ldx	#$00
	LONG	I,M			; set 16 bit register and memory mode
	pld				; restore direct page register
	rtl

LAB_08B5
	lda	$35
	bit	#$D008
	tsb	$2B
	rep	#$B0
	longa on
	longi on
	rtl

LAB_08BF
	pld				; restore direct page register
	ldy	#$E280
	bra	LAB_0887

;------------------------------------------------------------------------------------
;
; ??

;LAB_08C5
	.byte	$30,$6B

;------------------------------------------------------------------------------------
;
; high bytes for FCMP and FTST


LAB_08C7
	.byte	$48			; extended precision
	.byte	$54			; double precision
	.byte	$44			; single precision
	.byte	$40			; long integer
	.byte	$50			; word integer

;------------------------------------------------------------------------------------
;
; read/set FP status bits ??

LAB_08CC
	LDA	#$00A8		; FMOVE from FPSR
	STA	>FP_Cmd		; save command word
LAB_08D3
	LDA	>FP_Stat		; get FPU response code
	CMP	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	BEQ	LAB_08D3		; loop if FPU busy

	LDA	>FP_Oph		; get FPSR high word
	xba				; swap bytes
	sta	$24
	lda	>FP_Opl		; get FPSR low word
	xba				; swap bytes
	sta	$22
LAB_08EA
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_08EA		; loop if FPU busy

	stz	$1E
	lda	$22
	and	#$00F8
	sta	$30
	bne	LAB_08FF
	bra	LAB_0964

LAB_08FF
	and	#$0008
	beq	LAB_0909
	ldy	#$1000
	sty	$1E
LAB_0909
	lda	$30
	and	#$0010
	beq	LAB_0917
	lda	$1E
	ora	#$0800
	sta	$1E
LAB_0917
	lda	$30
	and	#$0020
	beq	LAB_0925
	lda	$1E
	ora	#$0200
	sta	$1E
LAB_0925
	lda	$30
	and	#$0040
	beq	LAB_0933
	lda	$1E
	ora	#$0400
	sta	$1E
LAB_0933
	lda	$30
	and	#$0080
	beq	LAB_0941
	lda	$1E
	ora	#$0100
	sta	$1E
LAB_0941
	lda	$1E
	xba				; swap bytes
	sta	$1E
	beq	LAB_0964
	lda	$1C
LAB_094A
	and	#$000F
	beq	LAB_0964
	xba				; swap bytes
	and	$1E
	beq	LAB_0964
	lda	$19
	sta	LAB_0960
	lda	$18
	sta	LAB_095F
LAB_095E
	jsl	>LAB_095E
	sta	$1E
LAB_0964
	lda	#$0088		; FMOVE to FPSR
	sta	>FP_Cmd		; save command word
LAB_096B
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_096B		; loop if FPU busy

	lda	#$0000
	sta	>FP_Oph
	sta	>FP_Opl
LAB_097F
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_097F		; loop if FPU busy

	lda	$1E
	ora	$20
	sta	$20
	lda	$1F
	and	#$001F
	sta	$21
	ldx	$20
	ldy	$21
	lda	$1E
	xba				; swap bytes
	ora	$1C
	sta	$1C
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; FRAND

LAB_09A9
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	brl	LAB_100D		; go do FRAND

;------------------------------------------------------------------------------------
;
; FEXP2M1

LAB_09B1
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	brl	LAB_0D47		; go do FEXP2M1

;------------------------------------------------------------------------------------
;
; FLOG2P1

LAB_09B9
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	brl	LAB_0CD8		; go do FLOG2P1

;------------------------------------------------------------------------------------
;
; ??

LAB_09C1
	phd				; push direct page register
	tcd				; transfer A to direct page register
	lda	$15,s
	sta	$16
	lda	$13,s
	sta	$14
	lda	$11,s
	sta	$12
	lda	$0F,s
	sta	$10
	lda	$0D,s
	sta	$0E
	lda	$0B,s
	sta	$0C
	lda	$09,s
	and	#$00FF
	sta	$0A
	lda	$07,s
	sta	$15,s
	lda	$05,s
	sta	$13,s
	lda	$03,s
	sta	$11,s
	pla
	sta	$0D,s
	pla
	pla
	pla
	pla
	pla
	pla
	lda	$0A
	cmp	#$0016
	bne	LAB_0A01
	brl	LAB_0E95

LAB_0A01
	brl	LAB_0DB8

;------------------------------------------------------------------------------------
;
; ??

LAB_0A04
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1BAF
	brl	LAB_10CD

;------------------------------------------------------------------------------------
;
; ??

LAB_0A0C
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_03E1
	brl	LAB_115C

;------------------------------------------------------------------------------------
;
; FCOPYSGN

LAB_0A14
	jsr	LAB_03E1
	SHORT	M			; set 8 bit memory mode
	lda	$2F			; get precision
	asl	a			; * 2 ??
	tax				; copy index for specified precision
	lda	LAB_0A42,x		; get index to sign bytes for specified precision
	tay				; copy to index
	lda	[$10],y		; get sign byte
	and	#$80			; mask sign bit
	beq	LAB_0A31		; branch if +ve

	ldy	#$0009		; index to sign
	lda	[$0C],y		; get sign byte
	ora	#$80			; set sign bit
	clc
	bcc	LAB_0A38

LAB_0A31
	ldy	#$0009		; index to sign
	lda	[$0C],y		; get sign byte
	and	#$7F			; clear sign bit
LAB_0A38
	sta	[$0C],y		; save sign byte
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; indeces to sign bytes for specified precision

LAB_0A42
	.byte	$09			; extended precision sign byte index
	.byte	$07			; double precision sign byte index
	.byte	$03			; single precision sign byte index
	.byte	$03			; long integer sign byte index
	.byte	$01			; word integer sign byte index

;------------------------------------------------------------------------------------
;
; ??

LAB_0A47
	jsr	LAB_03E1
	ldx	#$000B
	lda	#$0000
LAB_0A50
	sta	$61,x
	dex
	dex
	bpl	LAB_0A50
	SHORT	I,M			; set 8 bit register and memory mode
	ldy	#$05
	lda	[$10],y
	cmp	#$49
	bne	LAB_0A63
	brl	LAB_0B3E

LAB_0A63
	cmp	#$4E
	bne	LAB_0A6A
	brl	LAB_0B6C

LAB_0A6A
	LONG	I,M			; set 16 bit register and memory mode
	ldy	#$0002
	lda	[$10],y
	sta	$32
	iny
	iny
	lda	[$10],y
	and	#$00FF
	clc
	adc	$32
	sta	$32
	dec	$32
	lda	#$0050		; FMOVE to FPn, word integer
	sta	>FP_Cmd		; save command word
LAB_0A88
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0A88		; loop if FPU busy

	lda	$32
	xba				; swap bytes
	sta	>FP_Oph
LAB_0A98
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0A98		; loop if FPU busy

	lda	#$1200		; FTENTOX, source FP0
	sta	>FP_Cmd		; save command word
LAB_0AA7
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0AA7		; loop if FPU busy

	SHORT	I,M			; set 8 bit register and memory mode
	ldy	#$04
	lda	[$10],y
	bne	LAB_0ABA

	brl	LAB_0B9B

LAB_0ABA
	sta	$30
	iny
	lda	[$10],y
	and	#$0F
	ldx	#$03
	sta	$61,x
	inx
	dec	$30
	beq	LAB_0AEA
LAB_0ACA
	iny
	lda	[$10],y
	and	#$0F
	clc
	asl	a
	asl	a
	asl	a
	asl	a
	sta	$61,x
	dec	$30
	beq	LAB_0AEA
	iny
	lda	[$10],y
	and	#$0F
	ora	$61,x
	sta	$61,x
	inx
	dec	$30
	beq	LAB_0AEA
	bne	LAB_0ACA
LAB_0AEA
	LONG	I,M			; set 16 bit register and memory mode
	lda	#$234C		; FMUL, packed decimal
	sta	>FP_Cmd		; save command word
LAB_0AF3
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0AF3		; loop if FPU busy

	lda	$61
	sta	>FP_Oph
	lda	$63
	sta	>FP_Opl
	lda	$65
	sta	>FP_Oph
	lda	$67
	sta	>FP_Opl
	lda	$69
	sta	>FP_Oph
	lda	$6B
	sta	>FP_Opl
LAB_0B20
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0B20		; loop if FPU busy

	lda	[$10]
	beq	LAB_0B3B

	lda	#$1A00		; FNEG, source FP0
	sta	>FP_Cmd		; save command word
LAB_0B33
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0B33		; loop if FPU busy

LAB_0B3B
	brl	LAB_0B9B

LAB_0B3E
	LONG	I,M			; set 16 bit register and memory mode
	ldx	#$0000
	ldy	#$0008
LAB_0B46
	lda	LAB_0B62,x
	sta	[$0C],y
	inx
	inx
	dey
	dey
	bpl	LAB_0B46

	SHORT	I,M			; set 8 bit register and memory mode
	lda	[$10]
	bne	LAB_0B5F
	ldy	#$09
	lda	[$0C],y
	and	#$7F
	sta	[$0C],y
LAB_0B5F
	brl	LAB_0C46

;------------------------------------------------------------------------------------
;
; ??

LAB_0B62
	.byte	$FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00

;------------------------------------------------------------------------------------
;
; ??

LAB_0B6C
	LONG	I,M			; set 16 bit register and memory mode
	ldx	#$0000
	ldy	#$0008
LAB_0B74
	lda	LAB_0B90,x
	sta	[$0C],y
	inx
	inx
	dey
	dey
	bpl	LAB_0B74

	SHORT	I,M			; set 8 bit register and memory mode
	lda	[$10]
	bne	LAB_0B8D
	ldy	#$09
	lda	[$0C],y
	and	#$7F
	sta	[$0C],y
LAB_0B8D
	brl	LAB_0C46

;------------------------------------------------------------------------------------
;
; ??

LAB_0B90
	.byte	$FF,$FF
	.byte	$17,$40
	.byte	$00,$00
	.byte	$00,$00
	.byte	$00,$00
	.byte	$80,$E2
	.byte	$30,$A6
	.byte	$2F,$BD
	.byte	$50,$0C

;------------------------------------------------------------------------------------
;
; ??

;LAB_0BA2
	sta	>FP_Cmd		; save command word high byte
	lda	#$00			; FMOVE to FPn
	sta	>FP_Cmdl		; save command word low byte
	LONG	I,M			; set 16 bit register and memory mode
LAB_0BAE
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0BAE		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	tax
	bne	LAB_0BF3

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0008		; index to extended precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; second word is all 0s and is discarded

	lda	>FP_Oph		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0006		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Opl		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word

	brl	LAB_0C3E

LAB_0BF3
	dex
	bne	LAB_0C1E

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0006		; index to double precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word

	brl	LAB_0C3E

LAB_0C1E
	cpx	#$0003
	bne	LAB_0C2D
	lda	>FP_Oph
	xba				; swap bytes
	sta	[$0C]
	brl	LAB_0C3E

LAB_0C2D
	lda	>FP_Oph
	xba				; swap bytes
	ldy	#$0002
	sta	[$0C],y
	lda	>FP_Opl
	xba				; swap bytes
	sta	[$0C]
LAB_0C3E
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0C3E		; loop if FPU busy

LAB_0C46
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	I,M			; set 16 bit register and memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; ??

;LAB_0C50
	pla
	stz	$64,x
	rts

;------------------------------------------------------------------------------------
;
; ??

;LAB_0C54
	.byte	$70

;------------------------------------------------------------------------------------
;
; FDIV

LAB_0C55
	JSR	LAB_03E1

	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0C5F
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0C5F		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0C9E
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0C9E		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_0CCE,x		; get FDIV precision vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FDIV extended precision

LAB_0CB0
	lda	#$A048		; FDIV, extended precision, destination FP1
	brl	LAB_02A4		; do FPU command, extended precision source

;------------------------------------------------------------------------------------
;
; FDIV double precision

LAB_0CB6
	lda	#$A054		; FDIV, double precision, destination FP1
	brl	LAB_02ED		; do FPU command, double precision source

;------------------------------------------------------------------------------------
;
; FDIV single precision

LAB_0CBC
	lda	#$A044		; FDIV, single precision, destination FP1
	brl	LAB_0325		; do FPU command, single precision source

;------------------------------------------------------------------------------------
;
; FDIV long integer

LAB_0CC2
	lda	#$A040		; FDIV, long integer, destination FP1
	brl	LAB_0349		; do FPU command, long integer source

;------------------------------------------------------------------------------------
;
; FDIV word integer

LAB_0CC8
	lda	#$A050		; FDIV, word integer, destination FP1
	brl	LAB_036D		; do FPU command, word integer source

;------------------------------------------------------------------------------------
;
; FDIV precision vectors

LAB_0CCE
	.word	LAB_0CB0-1		; FDIV extended precision operation
	.word	LAB_0CB6-1		; FDIV double precision operation
	.word	LAB_0CBC-1		; FDIV single precision operation
	.word	LAB_0CC2-1		; FDIV long integer operation
	.word	LAB_0CC8-1		; FDIV word integer operation

;------------------------------------------------------------------------------------
;
; FLOG2P1
;
; FLOG2(1+n)

LAB_0CD8
	lda	#$B25C		; FMOVECR, destination FP1, offset $32, 10^0 decimal
	sta	>FP_Cmd		; save command word
LAB_0CDF
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0CDF		; loop if FPU busy

	lda	#$A248		; FADD, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0CEE
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0CEE		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0D2D
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0D2D		; loop if FPU busy

	lda	#$9604		; FLOG2, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0D3C
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0D3C		; loop if FPU busy

	brl	LAB_12F9		; go get extended result ??

;------------------------------------------------------------------------------------
;
; FOEXP21X
;
; 2^x-1

LAB_0D47
	lda	#$9148		; FTWOTOX, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0D4E
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0D4E		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0D8D
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0D8D		; loop if FPU busy

	lda	#$A850		; FSUB, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0D9C
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0D9C		; loop if FPU busy

	lda	#$0001		; 1
	xba				; swap bytes
	sta	>FP_Oph
LAB_0DAD
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0DAD		; loop if FPU busy

	brl	LAB_12F9		; go get extended result ??

;------------------------------------------------------------------------------------
;
; ??

LAB_0DB8
	lda	#$8050		; FMOVE to FPn, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0DBF
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0DBF		; loop if FPU busy

	lda	#$0001
	xba				; swap bytes
	sta	>FP_Oph
LAB_0DD0
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0DD0		; loop if FPU busy

	lda	#$A248		; FADD, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0DDF
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0DDF		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$14],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$14]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0E1E
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0E1E		; loop if FPU busy

	lda	#$9504		; FLOG10, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0E2D
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0E2D		; loop if FPU busy

	lda	#$A348		; FMUL, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0E3C
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0E3C		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0E7B
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0E7B		; loop if FPU busy

	lda	#$9204		; FTENTOX, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0E8A
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0E8A		; loop if FPU busy

	brl	LAB_12F9		; go get extended result ??

;------------------------------------------------------------------------------------
;
; ??

LAB_0E95
	lda	#$8050		; FMOVE to FPn, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0E9C
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0E9C		; loop if FPU busy

	lda	#$0001
	xba				; swap bytes
	sta	>FP_Oph
LAB_0EAD
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0EAD		; loop if FPU busy

	lda	#$A248		; FADD, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0EBC
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0EBC		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$14],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$14]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0EFB
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0EFB		; loop if FPU busy

	lda	#$9504		; FLOG10, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0F0A
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F0A		; loop if FPU busy

	lda	#$0049		; FMOVE to FPn, extended precision, destination FP2
	sta	>FP_Cmd		; save command word
LAB_0F19
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0F19		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_0F58
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F58		; loop if FPU busy

	lda	#$1A09		; FNEG, source FP2, destination FP2
	sta	>FP_Cmd		; save command word
LAB_0F67
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F67		; loop if FPU busy

	lda	#$A308		; FMUL, source FP2, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0F76
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F76		; loop if FPU busy

	lda	#$9204		; FTENTOX, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0F85
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F85		; loop if FPU busy

	lda	#$9A04		; FNEG, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0F94
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0F94		; loop if FPU busy

	lda	#$A250		; FADD, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0FA3
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0FA3		; loop if FPU busy

	lda	#$0001
	xba				; swap bytes
	sta	>FP_Oph
LAB_0FB4
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_0FB4		; loop if FPU busy

	lda	#$A048		; FDIV, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_0FC3
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_0FC3		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$14],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$14],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$14]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1002
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1002		; loop if FPU busy

	brl	LAB_12F9		; go get extended result ??

;------------------------------------------------------------------------------------
;
; FRAND, return an extended precision random number
;
; FRAND(n) = (16807 * n) MOD (2^31 - 1)

LAB_100D
	lda	#$8050		; FMOVE to FPn, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1014
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1014		; loop if FPU busy

	lda	#$41A7		; 16807d
	xba				; swap bytes
	sta	>FP_Oph
LAB_1025
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1025		; loop if FPU busy

	lda	#$A348		; FMUL, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1034
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1034		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1073
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1073		; loop if FPU busy

	lda	#$1151		; FTWOTOX, word integer, destination FP2
	sta	>FP_Cmd		; save command word
LAB_1082
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1082		; loop if FPU busy

	lda	#$001F		; 31d
	xba				; swap bytes
	sta	>FP_Oph
LAB_1093
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1093		; loop if FPU busy

	lda	#$2851		; FSUB, word integer, destination FP2
	sta	>FP_Cmd		; save command word
LAB_10A2
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_10A2		; loop if FPU busy

	lda	#$0001		; 1d
	xba				; swap bytes
	sta	>FP_Oph
LAB_10B3
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_10B3		; loop if FPU busy

	lda	#$A108		; FMOD, source FP2, destination FP1
	sta	>FP_Cmd		; save command word
LAB_10C2
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_10C2		; loop if FPU busy

	brl	LAB_12F9		; go get extended result ??

;------------------------------------------------------------------------------------
;
; ??

LAB_10CD
	ldy	#$0009		; index to sign
	lda	[$0C],y		; get sign
	and	#$0080		; mask sign bit
	beq	LAB_10DC		; if +ve just do FMOVE

	lda	#$9848		; else do FABS, extended precision, destination FP1
	bra	LAB_10DF		; go save command

LAB_10DC
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
LAB_10DF
	sta	>FP_Cmd		; save command word
LAB_10E3
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_10E3		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1122
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1122		; loop if FPU busy

	lda	#$9504		; FLOG10, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1131
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1131		; loop if FPU busy

	lda	#$0050		; FMOVE to FPn, word integer
	sta	>FP_Cmd		; save command word
LAB_1141
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1141		; loop if FPU busy

	lda	$10
	xba				; swap bytes
	sta	>FP_Oph
LAB_1151
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1151		; loop if FPU busy

	brl	LAB_1217

;------------------------------------------------------------------------------------
;
; ??

LAB_115C
	ldy	#$0009		; index to sign
	lda	[$0C],y		; get sign
	and	#$0080		; mask sign bit
	beq	LAB_116B		; if +ve just do FMOVE

	lda	#$9848		; else do FABS, extended precision, destination FP1
	bra	LAB_116E		; go save command

LAB_116B
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
LAB_116E
	sta	>FP_Cmd		; save command word
LAB_1172
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1172		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_11B1
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_11B1		; loop if FPU busy

	lda	#$9504		; FLOG10, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_11C0
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_11C0		; loop if FPU busy

	lda	#$0048		; FMOVE to FPn, extended precision
	sta	>FP_Cmd		; save command word
LAB_11D0
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_11D0		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_120F
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_120F		; loop if FPU busy

LAB_1217
	lda	#$A300		; FMUL, source FP0, destination FP1
	sta	>FP_Cmd		; save command word
LAB_121E
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_121E		; loop if FPU busy

	lda	#$9204		; FTENTOX, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_122E
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_122E		; loop if FPU busy

	lda	#$1800		; FABS, source FP0, destination FP0
	sta	>FP_Cmd		; save command word
LAB_123E
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_123E		; loop if FPU busy

	lda	#$0070		; FMOVE from FPn, word integer
	sta	>FP_Cmd		; save command word
LAB_124E
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_124E		; loop if FPU busy

	lda	>FP_Oph
	xba				; swap bytes
	sta	$30
LAB_125E
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_125E		; loop if FPU busy

	lda	$30
	bne	LAB_129A

	lda	#$325D		; FMOVECR, destination FP2, offset $32, 10^0 decimal
	sta	>FP_Cmd		; save command word
LAB_1271
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1271		; loop if FPU busy

	lda	#$2001		; FDIV, source FP0, destination FP2
	sta	>FP_Cmd		; save command word
LAB_1281
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1281		; loop if FPU busy

	lda	#$0008		; FMOVE to FPn, source FP2, destination FP0
	sta	>FP_Cmd		; save command word
LAB_1291
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1291		; loop if FPU busy

LAB_129A
	lda	#$2150		; FMOD, word integer
	sta	>FP_Cmd		; save command word
LAB_12A1
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_12A1		; loop if FPU busy

	lda	#$0200		; set integer word value
	sta	>FP_Oph		; save operand
LAB_12B1
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_12B1		; loop if FPU busy

	lda	#$0070		; FMOVE from FPn, word integer
	sta	>FP_Cmd		; save command word
LAB_12C0
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_12C0		; loop if FPU busy

	lda	>FP_Oph		; read result
	xba				; swap bytes
	tay
LAB_12CF
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_12CF		; loop if FPU busy

	tya
	and	#$0001
	bne	LAB_12DF

	bra	LAB_12F9		; go get extended result ??

LAB_12DF
	ldy	#$0009		; index to sign
	lda	[$0C],y		; get sign
	and	#$0080		; mask sign bit
	beq	LAB_12F9		; if +ve just go get extended result

	lda	#$9A04		; FNEG, source FP1, destination FP1
	sta	>FP_Cmd		; save command word
LAB_12F0
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_12F0		; loop if FPU busy

;------------------------------------------------------------------------------------
;
; get extended result ??

LAB_12F9
	lda	#$8068		; FMOVE from FPn, extended precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_1300
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1300		; loop if FPU busy

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0008		; index to extended precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; second word is all 0s and is discarded

	lda	>FP_Oph		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0006		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Opl		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word

LAB_133C
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_133C		; loop if FPU busy

	brl	LAB_1D6D

;------------------------------------------------------------------------------------
;
; ??

LAB_1347
	pla				; pull return address
	sta	$30			; save it

	lda	$0D,s
	sta	$0E
	lda	$0B,s
	sta	$0C			; save pointer to operand
	lda	$07,s
	sta	$0D,s
	sta	$04
	lda	$05,s
	sta	$0B,s
	sta	$02
	lda	$03,s
	sta	$09,s
	sta	$00
	pla
	sta	$05,s
	pla
	pla

	lda	$30			; get return address
	pha				; put it back on the stack
	rts

;------------------------------------------------------------------------------------
;
; FLOGN

LAB_136D
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$9448		; FLOGN, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FLOG2

LAB_1377
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$9648		; FLOG2, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FLOGNP1

LAB_1381
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$8648		; FLOGNP1, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FETOX

LAB_138B
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$9048		; FETOX, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FTWOTOX

LAB_1395
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$9148		; FTWOTOX, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FETOXM1

LAB_139F
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$8848		; FETOXM1, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; FATAN

LAB_13A9
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$8A48		; FATAN, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_13B3
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$8E48		; FSIN, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_13BD
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$9D48		; FCOS, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_13C7
	phd				; push direct page register
	tcd				; transfer A to direct page register
	jsr	LAB_1347
	lda	#$8F48		; FTAN, extended precision, destination FP1
	bra	LAB_13D1		; do FPU command and return extended result


;------------------------------------------------------------------------------------
;
; do FPU command and return extended result

LAB_13D1
	sta	>FP_Cmd		; save command word
LAB_13D5
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_13D5		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1414
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1414		; loop if FPU busy

	lda	#$8068		; FMOVE from FPn, extended precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_1423
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1423		; loop if FPU busy

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0008		; index to extended precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; second word is all 0s and is discarded

	lda	>FP_Oph		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0006		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Opl		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word

LAB_145F
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_145F		; loop if FPU busy

	brl	LAB_1D6D

;------------------------------------------------------------------------------------
;
; ??

LAB_146A
	stz	$20
	stz	$21
	lda	$1C
	sta	$20
	ldx	$20
	ldy	$21
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	lda	$05,s
	sta	$07,s
	lda	$03,s
	sta	$05,s
	lda	$01,s
	sta	$03,s
	pla
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_148D
	stz	$21
	lda	$18
	sta	$20
	lda	$19
	sta	$21
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	ldx	$20
	ldy	$21
	rtl

;------------------------------------------------------------------------------------
;
; FLOG2

LAB_14A5
	jsr	LAB_1347
	lda	#$9648		; FLOG2, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_14AF
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_14AF		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_14EE
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_14EE		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; FMUL

LAB_14F9
	jsr	LAB_03E1
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1503
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1503		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1542
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1542		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_1572,x		; get FMUL precision vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FMUL extended precision

LAB_1554
	lda	#$A348		; FMUL, extended precision, destination FP1
	brl	LAB_02A4		; do FPU command, extended precision source

;------------------------------------------------------------------------------------
;
; FMUL double precision

LAB_155A
	lda	#$A354		; FMUL, double precision, destination FP1
	brl	LAB_02ED		; do FPU command, double precision source

;------------------------------------------------------------------------------------
;
; FMUL single precision

LAB_1560
	lda	#$A344		; FMUL, single precision, destination FP1
	brl	LAB_0325		; do FPU command, single precision source

;------------------------------------------------------------------------------------
;
; FMUL long integer

LAB_1566
	lda	#$A340		; FMUL, long integer, destination FP1
	brl	LAB_0349		; do FPU command, long integer source

;------------------------------------------------------------------------------------
;
; FMUL word integer

LAB_156C
	lda	#$A350		; FMUL, word integer, destination FP1
	brl	LAB_036D		; do FPU command, word integer source

;------------------------------------------------------------------------------------
;
; FMUL precision vectors

LAB_1572
	.word	LAB_1554-1		; FMUL extended precision
	.word	LAB_155A-1		; FMUL double precision
	.word	LAB_1560-1		; FMUL single precision
	.word	LAB_1566-1		; FMUL long integer
	.word	LAB_156C-1		; FMUL word integer

;------------------------------------------------------------------------------------
;
; save operand of precision X
;
; X		precision
; -		---------
; $0000	extended
; $0001	double
; $0002	single
; $0003	long integer
; $0004	word integer

LAB_157C
	REP	#$30
LAB_157E
	LDA	>FP_Stat		; get FPU response code
	CMP	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	BEQ	LAB_157E		; loop if FPU busy

	CPX	#$0000		; compare with extended precision
	bne	LAB_15C5		; go try next if not extended precision

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

	brl	LAB_1611		; go wait for FPU and get status

LAB_15C5
	cpx	#$0001		; compare with double precision
	bne	LAB_15F2		; go try next if not double precision

	ldy	#$0006		; index to double precision sign and exponent
	lda	[$10],y		; get double precision sign and exponent
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	ldy	#$0004		; index to double precision high word
	lda	[$10],y		; get double precision high word
	xba				; swap bytes
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0002		; index to double precision next high word
	lda	[$10],y		; get double precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	lda	[$10]			; get double precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	brl	LAB_1611		; go wait for FPU and get status

LAB_15F2
	cpx	#$0004		; compare with word integer
	bne	LAB_1600		; go long integer/single precision if not word integer

	lda	[$10]			; get word integer
	xba				; swap bytes
	sta	>FP_Opl		; save FPU operand

	bra	LAB_1611		; go wait for FPU and get status

LAB_1600
	ldy	#$0002		; index to long integer/single precision high word
	lda	[$10],y		; get long integer/single precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save FPU operand high word

	lda	[$10]			; get long integer/single precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save FPU operand low word

LAB_1611
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1611		; loop if FPU busy

	lda	#$00A8		; FMOVE from FPSR
	sta	>FP_Cmd		; save command word
LAB_1620
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1620		; loop if FPU busy

	lda	>FP_Oph		; get FPSR high word
	xba				; swap bytes
	sta	$34			; save status high word

	lda	>FP_Opl		; get FPSR low word
	xba				; swap bytes
	sta	$32			; save status low word

LAB_1637
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1637		; loop if FPU busy

	rts

;------------------------------------------------------------------------------------
;
; FNEG

LAB_1640
	jsr	LAB_1347
	lda	#$9A48		; FNEG, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_164A
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_164A		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1689
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1689		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_1694
	jsr	LAB_1E25
	lda	$1C
	ldy	#$0000
	sta	[$0C],y
	sty	$1C
	sty	$1D
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_16AC
	jsr	LAB_1E25
	ldy	#$0000
	lda	$1C
	sta	[$0C],y
	lda	LAB_16BC
	brl	LAB_198D

;------------------------------------------------------------------------------------
;
; ??

LAB_16BC
	.byte	$00,$00

;------------------------------------------------------------------------------------
;
; ??

LAB_16BE
	jsr	LAB_1E25
	lda	$0C
	ora	$1E
	brl	LAB_094A

;------------------------------------------------------------------------------------
;
; ??

LAB_16C8
	jsr	LAB_1E25
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	lda	$1C
	and	$0C
	tax
	pld				; restore direct page register
	txa
	rtl

;------------------------------------------------------------------------------------
;
; FREM

LAB_16DB
	jsr	LAB_03E1
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_16E5
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_16E5		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1724
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1724		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_18B7,x		; get FREM precision vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FREM extended precision

LAB_1736
	lda	#$A548		; FREM, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_173D
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_173D		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

	brl	LAB_1816		; wait for FPU and get extended result ??

;------------------------------------------------------------------------------------
;
; FREM double precision

LAB_177F
	lda	#$A554		; FREM, double precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1786
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1786		; loop if FPU busy

	ldy	#$0006		; index to double precision high word
	lda	[$10],y		; get double precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	ldy	#$0004		; index to double precision next high word
	lda	[$10],y		; get double precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0002		; index to double precision next high word
	lda	[$10],y		; get double precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	lda	[$10]			; get double precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	brl	LAB_1816		; wait for FPU and get extended result ??

;------------------------------------------------------------------------------------
;
; FREM single precision

LAB_17B7
	lda	#$A544		; FREM, single precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_17BE
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_17BE		; loop if FPU busy

	ldy	#$0002		; index to single precision high word
	lda	[$10],y		; get single precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand high word

	lda	[$10]			; get single precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save to FPU operand low word

	brl	LAB_1816		; wait for FPU and get extended result ??

;------------------------------------------------------------------------------------
;
; FREM long integer

LAB_17DB
	lda	#$A540		; FREM, long integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_17E2
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_17E2		; loop if FPU busy

	ldy	#$0002		; set index to high word
	lda	[$10],y		; get long integer high word
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand high word

	lda	[$10]			; get long integer low word
	xba				; swap bytes
	sta	>FP_Opl		; save to FPU operand low word
	brl	LAB_1816		; wait for FPU and get extended result ??

;------------------------------------------------------------------------------------
;
; FREM word integer

LAB_17FF
	lda	#$A550		; FREM, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1806
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1806		; loop if FPU busy

	lda	[$10]			; get word integer source
	xba				; swap bytes
	sta	>FP_Oph		; save to FPU operand word

;------------------------------------------------------------------------------------
;
; wait for FPU and get extended result ??

LAB_1816
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1816		; loop if FPU busy

	lda	#$8068		; FMOVE from FPn, extended precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_1825
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1825		; loop if FPU busy

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0008		; set index to sign and exponent
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; second word is all 0s and is discarded

	lda	>FP_Oph		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0006		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Opl		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word
LAB_1861
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1861		; loop if FPU busy

	lda	#$00A8		; FMOVE from FPSR
	sta	>FP_Cmd		; save command word
LAB_1870
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1870		; loop if FPU busy

	lda	>FP_Oph		; get FPSR high word
	sta	$32
	lda	>FP_Opl		; get FPSR low word
	sta	$30
LAB_1885
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1885		; loop if FPU busy

	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	I,M			; set 16 bit register and memory mode
	ldy	#$0000
	lda	$32
	and	#$FF00
	tax
	sta	$20
	asl	a
	bcc	LAB_18B3
	lda	$20
	and	#$7F00
	tax
	sta	$20
	lda	#$0080
	tay
	sta	$21
	sep	#$80
LAB_18B3
	pld				; restore direct page register
	rtl

;LAB_18B5
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; FREM precision vectors

LAB_18B7
	.word	LAB_1736-1		; FREM extended precision
	.word	LAB_177F-1		; FREM double precision
	.word	LAB_17B7-1		; FREM single precision
	.word	LAB_17DB-1		; FREM long integer
	.word	LAB_17FF-1		; FREM word integer

;------------------------------------------------------------------------------------
;
; FINT

LAB_18C1
	JSR	LAB_1347
	lda	#$8148		; FINT, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_18CB
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_18CB		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_190A
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_190A		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_1915
	jsr	LAB_1BAF
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_191F
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_191F		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_195E
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_195E		; loop if FPU busy

	lda	#$A650		; FSCALE, word integer, destination FP1
	sta	>FP_Cmd		; save command word
LAB_196D
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_196D		; loop if FPU busy

	lda	$10
	xba				; swap bytes
	sta	>FP_Oph
LAB_197D
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_197D		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_1988
	jsr	LAB_1A7B
	bra	LAB_198F

LAB_198D
	sta	$0C
LAB_198F
	lda	$1C
	cmp	$0C
	bne	LAB_1998
	brl	LAB_1A08

LAB_1998
	stz	$26
	lda	$0C
	sta	$1C
	and	#$C000
	beq	LAB_19C6
	cmp	#$C000
	bne	LAB_19B1
	lda	$26
	ora	#$0010
	sta	$26
	bra	LAB_19C6

LAB_19B1
	cmp	#$4000
	bne	LAB_19BF
	lda	$26
	ora	#$0030
	sta	$26
	bra	LAB_19C6

LAB_19BF
	lda	$26
	ora	#$8000
	sta	$26
LAB_19C6
	lda	$1C
	and	#$C000
	beq	LAB_19E2
	cmp	#$0080
	bne	LAB_19DB
	lda	$26
	ora	#$0040
	sta	$26
	bra	LAB_19E2

LAB_19DB
	lda	$26
	ora	#$0080
	sta	$26
LAB_19E2
	lda	#$0090		; FMOVE to FPCR
	sta	>FP_Cmd		; save command word
LAB_19E9
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_19E9		; loop if FPU busy

	lda	#$0000
	sta	>FP_Oph
	lda	$26
	xba				; swap bytes
	sta	>FP_Opl
LAB_1A00
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1A00		; loop if FPU busy

LAB_1A08
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_1A12
	jsr	LAB_1347
	lda	$0C
	sta	$18
	lda	$0E
	sta	$1A
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; FSQRT

LAB_1A27
	jsr	LAB_1347
	lda	#$8448		; FSQRT, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1A31
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1A31		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1A70
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1A70		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

;------------------------------------------------------------------------------------
;
; ??

LAB_1A7B
	pla
	sta	$30
	lda	$0B,s
	sta	$0C
	lda	$07,s
	sta	$0B,s
	sta	$04
	lda	$05,s
	sta	$09,s
	sta	$02
	lda	$03,s
	sta	$07,s
	sta	$00
	pla
	sta	$03,s
	pla
	lda	$30
	pha
	rts

;------------------------------------------------------------------------------------
;
; copyright notice

;LAB_1A9C
	.byte	"Copyright (C) Innovative Systems 1988,89All rights reserved."

;------------------------------------------------------------------------------------
;
; FSUB

LAB_1AD8
	JSR	LAB_03E1
 	LDA	#$8048		; FMOVE to FPn, extended precision, destination FP1
	STA	>FP_Cmd		; save command word
LAB_1AE2
	LDA	>FP_Stat		; get FPU response code
	CMP	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	BEQ	LAB_1AE2		; loop if FPU busy

	LDY	#$0008		; index to extended precision high word
	LDA	[$0C],y		; get extended precision high word
	XBA				; swap bytes
	STA	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1B21
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1B21		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_1B51,x		; get FSUB precision vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FSUB extended precision

LAB_1B33
	lda	#$A848		; FSUB, extended precision, destination FP1
	brl	LAB_02A4		; do FPU command, extended precision source

;------------------------------------------------------------------------------------
;
; FSUB double precision

LAB_1B39
	lda	#$A854		; FSUB, double precision, destination FP1
	brl	LAB_02ED		; do FPU command, double precision source

;------------------------------------------------------------------------------------
;
; ??

LAB_1B3F
	lda	#$A844		; FSUB, single precision, destination FP1
	brl	LAB_0325		; do FPU command, single precision source

;------------------------------------------------------------------------------------
;
; FSUB long integer

LAB_1B45
	lda	#$A840		; FSUB, long integer, destination FP1
	brl	LAB_0349		; do FPU command, long integer source

;------------------------------------------------------------------------------------
;
; FSUB word integer

LAB_1B4B
	lda	#$A850		; FSUB, word integer, destination FP1
	brl	LAB_036D		; do FPU command, word integer source

;------------------------------------------------------------------------------------
;
; FSUB precision vectors

LAB_1B51
	.word	LAB_1B33-1		; FSUB extended precision
	.word	LAB_1B39-1		; FSUB double precision
	.word	LAB_1B3F-1		; FSUB single precision
	.word	LAB_1B45-1		; FSUB long integer 
	.word	LAB_1B4B-1		; FSUB word integer

;------------------------------------------------------------------------------------
;
; FINTRZ ??

LAB_1B5B
	jsr	LAB_1347
	lda	#$8348		; FINTRZ, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1B65
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1B65		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$0C],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$0C],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$0C]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1BA4
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1BA4		; loop if FPU busy

	brl	LAB_0381		; wait for FPU and get extended result

LAB_1BAF
	pla
	sta	$30
	lda	$0F,s
	sta	$10
	lda	#$0000
	sta	$12
	lda	$0B,s
	sta	$0C
	lda	$0D,s
	sta	$0E
	lda	$07,s
	sta	$0F,s
	sta	$04
	lda	$05,s
	sta	$0D,s
	sta	$02
	lda	$03,s
	sta	$0B,s
	sta	$00
	pla
	sta	$07,s
	pla
	pla
	pla
	lda	$30
	pha
	rts

;------------------------------------------------------------------------------------
;
; FX2Z
;
; convert extended precision to specified precision

LAB_1BDF
	jsr	LAB_03E1
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	sta	>FP_Cmd		; save command word
LAB_1BE9
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1BE9		; loop if FPU busy

	ldy	#$0008		; index to extended precision high word
	lda	[$10],y		; get extended precision high word
	xba				; swap bytes
	sta	>FP_Oph		; save first FPU operand high word

	lda	#$0000		; first FPU operand low word is all 0s
	sta	>FP_Opl		; save first FPU operand low word

	ldy	#$0006		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save second FPU operand high word

	ldy	#$0004		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Opl		; save second FPU operand low word

	ldy	#$0002		; index to extended precision next high word
	lda	[$10],y		; get extended precision next high word
	xba				; swap bytes
	sta	>FP_Oph		; save third FPU operand high word

	lda	[$10]			; get extended precision low word
	xba				; swap bytes
	sta	>FP_Opl		; save third FPU operand low word

LAB_1C28
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1C28		; loop if FPU busy

	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_1CF4,x		; get return result from FP1 vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; get double precision and ??

LAB_1C3A
	lda	#$8074		; FMOVE from FPn, double precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_1C41
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1C41		; loop if FPU busy

	lda	>FP_Oph		; get sign and exponent
	xba				; swap bytes
	ldy	#$0006		; index to double precision high word
	sta	[$0C],y		; save sign and exponent

	lda	>FP_Opl		; get most significant mantissa word
	xba				; swap bytes
	ldy	#$0004		; set index to most significant mantissa word
	sta	[$0C],y		; save most significant mantissa word

	lda	>FP_Oph		; get next most significant mantissa word
	xba				; swap bytes
	ldy	#$0002		; set index to next most significant mantissa word
	sta	[$0C],y		; save next most significant mantissa word

	lda	>FP_Opl		; get least significant mantissa word
	xba				; swap bytes
	sta	[$0C]			; save least significant mantissa word

LAB_1C6F
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1C6F		; loop if FPU busy

	brl	LAB_08CC		; read/set FP status bits ??

;------------------------------------------------------------------------------------
;
; get single precision and ??

LAB_1C7A
	lda	#$8064		; FMOVE from FPn, single precision, source FP1
	sta	>FP_Cmd		; save command word
LAB_1C81
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1C81		; loop if FPU busy

	lda	>FP_Oph
	xba				; swap bytes
	ldy	#$0002
	sta	[$0C],y
	lda	>FP_Opl
	xba				; swap bytes
	sta	[$0C]
LAB_1C9B
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1C9B		; loop if FPU busy

	brl	LAB_08CC		; read/set FP status bits ??

;------------------------------------------------------------------------------------
;
; get long integer and ??

LAB_1CA6
	lda	#$8060		; FMOVE from FPn, long integer, source FP1
	sta	>FP_Cmd		; save command word
LAB_1CAD
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1CAD		; loop if FPU busy

	lda	>FP_Oph		; get long integer high word
	xba				; swap bytes
	ldy	#$0002		; set index to high word
	sta	[$0C],y		; save long integer high word

	lda	>FP_Opl		; get long integer low word
	xba				; swap bytes
	sta	[$0C]			; save long integer low word
LAB_1CC7
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1CC7		; loop if FPU busy

	brl	LAB_08CC		; read/set FP status bits ??

;------------------------------------------------------------------------------------
;
; get word integer and ??

LAB_1CD2
	lda	#$8070		; FMOVE from FPn, word integer, source FP1
	sta	>FP_Cmd		; save command word
LAB_1CD9
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1CD9		; loop if FPU busy

	lda	>FP_Oph		; get word integer
	xba				; swap bytes
	sta	[$0C]			; save word integer result
LAB_1CE9
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1CE9		; loop if FPU busy

	brl	LAB_08CC		; read/set FP status bits ??

;------------------------------------------------------------------------------------
;
; return result from FP1

LAB_1CF4
	.word	LAB_1D0B-1		; get extended precision and ??
	.word	LAB_1C3A-1		; get double precision and ??
	.word	LAB_1C7A-1		; get single precision and ??
	.word	LAB_1CA6-1		; get long integer and ??
	.word	LAB_1CD2-1		; get word integer and ??

;------------------------------------------------------------------------------------
;
; FMOVE to FP1

LAB_1CFE
	jsr	LAB_03E1
	lda	$2E
	xba				; swap bytes
	asl	a			; * 2 bytes per vector
	tax				; copy to index
	lda	LAB_1D29,x		; get FMOVE to FP1 vector
	pha				; put on stack
	rts				; execute the vector

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 extended precision

LAB_1D0B
	lda	#$8048		; FMOVE to FPn, extended precision, destination FP1
	brl	LAB_02A4		; do FPU command, extended precision source

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 double precision

LAB_1D11
	lda	#$8054		; FMOVE to FPn, double precision, destination FP1
	brl	LAB_02ED		; do FPU command, double precision source

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 single precision

LAB_1D17
	lda	#$8044		; FMOVE to FPn, single precision, destination FP1
	brl	LAB_0325		; do FPU command, single precision source

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 long integer

LAB_1D1D
	lda	#$8040		; FMOVE to FPn, long integer, destination FP1
	brl	LAB_0349		; do FPU command, long integer source

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 word integer

LAB_1D23
	lda	#$8050		; FMOVE to FPn, word integer, destination FP1
	brl	LAB_036D		; do FPU command, word integer source

;------------------------------------------------------------------------------------
;
; FMOVE to FP1 vectors

LAB_1D29
	.word	LAB_1D0B-1		; FMOVE to FP1 extended precision
	.word	LAB_1D11-1		; FMOVE to FP1 double precision
	.word	LAB_1D17-1		; FMOVE to FP1 single precision
	.word	LAB_1D1D-1		; FMOVE to FP1 long integer
	.word	LAB_1D23-1		; FMOVE to FP1 word integer

;------------------------------------------------------------------------------------
;
; ??

LAB_1D33
	PLA
	STA	$30
	LDA	$15,s
	STA	$16
	LDA	$13,s
	STA	$14
	LDA	$11,s
	STA	$12
	LDA	$0F,s
	STA	$10
	LDA	$0D,s
	STA	$0E

	lda	$0B,s
	sta	$0C
	lda	$07,s
	sta	$15,s
	sta	$04
	lda	$05,s
	sta	$13,s
	sta	$02
	lda	$03,s
	sta	$11,s
	sta	$00
	pla
	sta	$0D,s
	pla
	pla
	pla
	pla
	pla
	pla
	lda	$30
	pha
	rts

;------------------------------------------------------------------------------------
;
; ??

LAB_1D6D
	lda	#$00A8		; FMOVE from FPSR
	sta	>FP_Cmd		; save command word
LAB_1D74
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1D74		; loop if FPU busy

	lda	>FP_Oph		; get FPSR high word
	xba				; swap bytes
	sta	$24
	lda	>FP_Opl		; get FPSR low word
	xba				; swap bytes
	sta	$22
LAB_1D8B
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1D8B		; loop if FPU busy

	stz	$1E
	lda	$22
	and	#$00F8
	sta	$30
	beq	LAB_1E05
	and	#$0008
	beq	LAB_1DAA
	lda	#$1000
	ora	$1E
	sta	$1E
LAB_1DAA
	lda	$30
	and	#$0010
	beq	LAB_1DB8
	lda	$1E
	ora	#$0800
	sta	$1E
LAB_1DB8
	lda	$30
	and	#$0020
	beq	LAB_1DC6
	lda	$1E
	ora	#$0200
	sta	$1E
LAB_1DC6
	lda	$30
	and	#$0040
	beq	LAB_1DD4
	lda	$1E
	ora	#$0400
	sta	$1E
LAB_1DD4
	lda	$30
	and	#$0080
	beq	LAB_1DE2
	lda	$1E
	ora	#$0100
	sta	$1E
LAB_1DE2
	lda	#$0088		; FMOVE to FPSR
	sta	>FP_Cmd		; save command word
LAB_1DE9
	lda	>FP_Stat		; get FPU response code
	cmp	#$0089		; NULL CA=1 PC=0 IA=1 PF=0 TF=0
	beq	LAB_1DE9		; loop if FPU busy

	lda	#$0000
	sta	>FP_Oph
	sta	>FP_Opl
LAB_1DFD
	lda	>FP_Stat		; get FPU response code
	xba				; swap bytes
	asl	a			; shift CA bit to Cb
	bcs	LAB_1DFD		; loop if FPU busy

LAB_1E05
	lda	$1C
	ora	$1E
	sta	$1C
	lda	$1E
	xba				; swap bytes
	sta	$1E
	ora	$20
	sta	$20
	lda	$1F
	and	#$001F
	sta	$21
	SHORT	M			; set 8 bit memory mode
	lda	$FA
	pha
	plb
	LONG	M			; set 16 bit memory mode
	pld				; restore direct page register
	rtl

;------------------------------------------------------------------------------------
;
; ??

LAB_1E25
	plx
	ply
	lda	$09,s
	sta	$0C
	lda	$05,s
	sta	$09,s
	lda	$03,s
	sta	$07,s
	lda	$01,s
	sta	$05,s
	pla
	pla
	phy
	phx
	rts

;------------------------------------------------------------------------------------

	end

